/*
 * shmdemo2.c
 * Written by Richard Dawe <richdawe@bigfoot.com>
 *
 * Demonstrate the POSIX-y shared memory functions.
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <fcntl.h>
#include <errno.h>

#include "posixshm.h"

#define SHMDEMO2_SHM_NAME "/foo/bar/shmdemo2"

char SHMDEMO2_DATA[] = "Hello world!\nOh my gosh it works!";
char buf[1024];

void die (void) { perror("shmdemo2"); exit(EXIT_FAILURE); }

int main (void)
{
	int fd1, fd2, ret;

	/* Until the permissions are sorted out, pass 0 as mode. */
	fd1 = shm_open(SHMDEMO2_SHM_NAME, O_CREAT, 0);
	if (fd1 == -1) die();
	printf(". shm segment %s created\n", SHMDEMO2_SHM_NAME);

	ret = lseek(fd1, 0, SEEK_SET);
	printf(". fd1 seeked to %u\n", ret);
	ret = write(fd1, (void *) SHMDEMO2_DATA, strlen(SHMDEMO2_DATA) + 1);
	if (ret == -1) die();
	printf(". Wrote %i bytes to fd1\n", ret);

	/* Open another reference */
	fd2 = shm_open(SHMDEMO2_SHM_NAME, 0, 0);
	if (fd2 == -1) die();
	printf(". shm segment %s opened\n", SHMDEMO2_SHM_NAME);

	ret = lseek(fd2, 0, SEEK_SET);
	printf(". fd2 seeked to %u\n", ret);
	ret = read(fd2, (void *) buf, sizeof(buf));
	if (ret == -1) die();
	printf(". Read %i bytes from fd2\n", ret);
	printf(". Message '%s'\n", buf);

	close(fd2);
	close(fd1);

	/* Test whether the shm segment is still there. */
	fd2 = shm_open(SHMDEMO2_SHM_NAME, 0, 0);
	if (fd2 == -1) die();
	printf(". shm segment %s re-opened\n", SHMDEMO2_SHM_NAME);

	ret = lseek(fd2, 0, SEEK_SET);
	printf(". fd2 seeked to %u\n", ret);
	ret = read(fd2, (void *) buf, sizeof(buf));
	if (ret == -1) die();
	printf(". Read %i bytes from fd2\n", ret);
	printf(". Message '%s'\n", buf);

	close(fd2);

	/* Try deleting the segment */
	ret = shm_unlink(SHMDEMO2_SHM_NAME);
	if (ret == -1) die();
	printf(". shm segment %s destroyed\n", SHMDEMO2_SHM_NAME);

	/* Try to re-open the segment */
	fd1 = shm_open(SHMDEMO2_SHM_NAME, 0, 0);
	if (fd1 == -1) {
		printf(". shm_open() on dead segment returned error "
		       "(expected):\n"
		       "  errno = %i (%s)\n", errno, strerror(errno));
	}

	return(EXIT_SUCCESS);
}
